local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MagicDKP", false)

--------------------------------------------------
-- Item display => Based on the Label AceGuiWidget
-- It pretends to be an "input"...
--------------------------------------------------
do
	local Type = "MDKPItemWidget"
	local Version = 1
	
	local function Acquire(self)
		self:SetText("")
		self:SetImage(nil)
	end
	
	local function Release(self)
		self.frame:ClearAllPoints()
		self.frame:Hide()
	end
	
	local function UpdateImageAnchor(self)
	   local width = self.frame.width or self.frame:GetWidth() or 0
	   local image = self.image
	   local label = self.label
	   local frame = self.frame
	   local height
	   
	   label:ClearAllPoints()
	   image:ClearAllPoints()
	   
	   if self.imageshown then
	      local imagewidth = image:GetWidth()
	      if (width - imagewidth) < 200 or (label:GetText() or "") == "" then
		 --image goes on top centered when less than 200 width for the text, or if there is no text
		 image:SetPoint("TOP",frame,"TOP",0,0)
		 label:SetPoint("TOP",image,"BOTTOM",0,0)
		 label:SetPoint("LEFT",frame,"LEFT",0,0)
		 label:SetWidth(width)
		 height = image:GetHeight() + label:GetHeight()
	      else
		 --image on the left
		 image:SetPoint("TOPLEFT",frame,"TOPLEFT",0,0)
		 label:SetPoint("TOPLEFT",image,"TOPRIGHT",3,0)
		 label:SetWidth(width - imagewidth-3)
		 height = math.max(image:GetHeight(), label:GetHeight())
	      end
	   else
	      --no image shown
	      label:SetPoint("TOPLEFT",frame,"TOPLEFT",0,0)
	      label:SetWidth(width)
	      height = self.label:GetHeight()
	   end
	   
	   frame.resizing = true
	   self.frame:SetHeight(height)
	   self.frame.height = height
	   frame.resizing = nil
	end
	
	local function SetText(self, text, itemid)
	   if itemid then
	      self.itemid = itemid
	   else
	      self.itemid, text = string.match(text, "(%d+):(.+)")
	   end
	   self.label:SetText(text or "")

	   if self.itemid then
	      local image = select(10, GetItemInfo(self.itemid))
	      if image then self:SetImage(image) end
	   end
	   UpdateImageAnchor(self)
	end
	
	local function OnWidthSet(self, width)
	   UpdateImageAnchor(self)
	end
	
	local function OnFrameResize(this)
	   if this.resizing then return end
	   local self = this.obj
	   OnWidthSet(self, this:GetWidth())
	end

	local function SetImage(self, path, ...)
	   local image = self.image
	   image:SetTexture(path)

	   if image:GetTexture() then
	      self.imageshown = true
	      local n = select('#', ...)
	      if n == 4 or n == 8 then
		 image:SetTexCoord(...)
	      end
	   else
	      self.imageshown = nil
	   end
	   UpdateImageAnchor(self)
	end
	
	local function SetLabel(self, texture)
	   self:SetImage(texture)
	   self:SetImageSize(24, 24)
	end

	local function Button_OnClick(this)
	   local self = this.obj

	   if not self.itemid then return end
	   
	   if ( IsShiftKeyDown() and ChatFrame1EditBox:IsVisible() ) then
	      local _, itemLink = GetItemInfo(self.itemid)
	      ChatFrame1EditBox:Insert(itemLink)
	   end
	end
	local function Button_OnMouseUp(this)
	   local self = this.obj
	   
	   if arg1 == "RightButton" and self.itemid then
	      GameTooltip:SetOwner(this, "ANCHOR_CURSOR")
	      GameTooltip:SetHyperlink("item:"..self.itemid)
	      GameTooltip:Show()
	   end
	end


	local function Button_OnEnter(this)
	   local self = this.obj
	   if self.itemid and tonumber(self.itemid) > 0 then
	      GameTooltip:SetOwner(this, "ANCHOR_CURSOR");
	      if GetItemInfo(self.itemid) then
		 GameTooltip:SetHyperlink("item:"..self.itemid)
	      else
		 local item = MagicDKPDB.items[tonumber(self.itemid)]
		 GameTooltip:AddLine(gsub(gsub(item.itemInfo[2], "[\[]", ""), "[\]]", ""))
		 GameTooltip:AddLine(L["This item is not in your cache. You can try to get the information by"], 1, 1, 1);
		 GameTooltip:AddLine(L["rightclicking the item. Note that this may result in a disconnect!"], 1, 1, 1);
	      end
	      GameTooltip:Show()
	   end
	end

	local function SetImageSize(self, width, height)
		self.image:SetWidth(width)
		self.image:SetHeight(height or width)
		self.label:SetHeight(height or width)
		UpdateImageAnchor(self)
	end

	local function Constructor()
		local frame = CreateFrame("Button",nil,UIParent)
		local self = {}
		self.type = Type

		self.Release = Release
		self.Acquire = Acquire

		self.SetText = SetText
		self.SetLabel = SetLabel
		self.frame = frame
		self.OnWidthSet = OnWidthSet
		self.SetImage = SetImage
		self.SetImageSize = SetImageSize

		frame.obj = self
		
		frame:SetHeight(18)
		frame:SetWidth(200)
		frame:SetScript("OnSizeChanged", OnFrameResize)

		local label = frame:CreateFontString(nil,"BACKGROUND","GameFontHighlight")
		label:SetPoint("TOPLEFT",frame,"TOPLEFT",0,0)
		label:SetWidth(200)
		label:SetJustifyH("LEFT")
		label:SetJustifyV("CENTER")
		self.label = label
		
		local image = frame:CreateTexture(nil,"BACKGROUND")
		self.image = image
		frame:SetScript("OnEnter", Button_OnEnter)
		frame:SetScript("OnLeave", function() GameTooltip:Hide() end)
		frame:SetScript("OnClick", Button_OnClick)
		frame:SetScript("OnMouseUp", Button_OnMouseUp)
		self:SetImageSize(32)
		AceGUI:RegisterAsWidget(self)
		return self
	end
	
	AceGUI:RegisterWidgetType(Type,Constructor,Version)
end

